'use strict';

exports.__esModule = true;
exports.createClient = createClient;

var _client = require('./modern/client');

var _client2 = require('./legacy/client');

/**
 * Constructs a new Client instance with the given scriptURL
 *
 * @param  {string} scriptURL The location of your server script
 * @param  {Object} options   Initial options
 * @return {MockerClient}
 */
function createClient(scriptURL) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  if (options.forceLegacy) {
    return new _client2.LegacyClient(scriptURL);
  }

  var useLegacy = isLegacyMode();

  /* istanbul ignore if */
  if (useLegacy) {
    console.warn('Switching to legacy mode...');
    return new _client2.LegacyClient(scriptURL);
  }

  return new _client.ModernClient(scriptURL);
}

/* istanbul ignore next */
function isLegacyMode() {
  if (!('serviceWorker' in navigator)) {
    // tslint:disable-next-line max-line-length
    console.warn('Service worker is not supported in your browser, please check: http://caniuse.com/#feat=serviceworkers');

    return true;
  }

  if (location.protocol !== 'https:' && location.hostname !== 'localhost' && location.hostname !== '127.0.0.1') {
    // tslint:disable-next-line max-line-length
    console.warn('Service workers should be registered in secure pages, further information: https://github.com/w3c/ServiceWorker/blob/master/explainer.md#getting-started');

    return true;
  }

  return false;
}
//# sourceMappingURL=index.js.map