'use strict';

exports.__esModule = true;
exports.LegacyClient = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _utils = require('../../utils/');

var _constants = require('../../constants/');

var _patchXhr = require('./patch-xhr');

var _patchFetch = require('./patch-fetch');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var clientLog = _utils.debug.scope('legacy');
var registrations = {};

var LegacyClient = exports.LegacyClient = function () {

  /**
   * Points to currently activated ServiceWorker object.
   * Stays null when running in legacy mode.
   *
   * @readonly
   * @type {null}
   */

  /**
   * Indicates which mode current client is running on.
   *
   * @readonly
   * @type {boolean}
   */
  function LegacyClient(scriptURL) {
    var _this = this;

    _classCallCheck(this, LegacyClient);

    this.isLegacy = true;
    this.ready = null;
    this.controller = null;
    this._registration = null;

    (0, _patchXhr.patchXHR)();
    (0, _patchFetch.patchFetch)();

    var promise = null;

    // avoid duplications
    if (registrations[scriptURL]) {
      promise = registrations[scriptURL];
    } else {
      registrations[scriptURL] = promise = new Promise(function (resolve, reject) {
        var script = document.createElement('script');
        script.src = scriptURL;
        script.onload = resolve;
        script.onerror = reject;

        document.body.appendChild(script);
      });
    }

    /* istanbul ignore next */
    this.ready = promise.then(function () {
      return (0, _utils.sendMessageRequest)(window, {
        action: _constants.ACTION.PING
      });
    }).then(function () {
      clientLog.info('connection established');
      return _this._registration;
    }).catch(function (error) {
      // `ready` should never be rejected
      clientLog.error('bootstrap failed', error);
    });
  }

  /**
   * Update registration, this method has no effect in legacy mode
   *
   * @return {Promise<null>}
   */


  /**
   * Defines whether a client has connected to mocker server.
   * Resolves with `null` as there're actually no registrations
   *
   * @readonly
   * @type {Promise<null>}
   */


  LegacyClient.prototype.update = function () {
    var _ref = _asyncToGenerator(_regenerator2.default.mark(function _callee() {
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              return _context.abrupt('return', Promise.resolve(this._registration));

            case 1:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function update() {
      return _ref.apply(this, arguments);
    }

    return update;
  }();

  /**
   * Get current registration, resolved with `null` in legacy mode
   *
   * @return {Promise<null>}
   */


  LegacyClient.prototype.getRegistration = function () {
    var _ref2 = _asyncToGenerator(_regenerator2.default.mark(function _callee2() {
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              return _context2.abrupt('return', Promise.resolve(this._registration));

            case 1:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    function getRegistration() {
      return _ref2.apply(this, arguments);
    }

    return getRegistration;
  }();

  /* istanbul ignore next */
  /**
   * Unregister mocker, this method has no effect in legacy mode
   *
   * @return {Promise<false>}
   */


  LegacyClient.prototype.unregister = function () {
    var _ref3 = _asyncToGenerator(_regenerator2.default.mark(function _callee3() {
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _utils.debug.scope('legacy').warn('mocker in legacy mode can\'t be unregistered');

              return _context3.abrupt('return', false);

            case 2:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, this);
    }));

    function unregister() {
      return _ref3.apply(this, arguments);
    }

    return unregister;
  }();

  return LegacyClient;
}();
//# sourceMappingURL=client.js.map