'use strict';

exports.__esModule = true;
exports.dispatchFetchEvent = dispatchFetchEvent;

var _utils = require('../../utils/');

var _createEvent = require('./create-event');

var fetchEvents = [];
var addEventListener = self.addEventListener.bind(self);

// handle fetch events ourselves
self.addEventListener = function (type, listener, useCapture) {
  if (type === 'fetch') {
    fetchEvents.push(listener);
  } else {
    addEventListener(type, listener, useCapture);
  }
};

/**
 * Dispatch fetch event on GlobalScope in legacy mode.
 * Resolved with `null` if `event.respondWith` isn't called.
 *
 * @param  {Request} request Request object
 * @return {Promise<Response>}
 */
function dispatchFetchEvent(request) {
  var fetchEvt = (0, _createEvent.createEvent)('fetch');
  var deferred = new _utils.Defer();

  fetchEvt.request = request;

  fetchEvt.respondWith = function (response) {
    if (deferred.done) {
      // tslint:disable-next-line max-line-length
      throw new Error('Failed to execute \'respondWith\' on \'FetchEvent\': The fetch event has already been responded to.');
    }

    deferred.resolve(response);
  };

  fetchEvents.forEach(function (listener) {
    listener(fetchEvt);
  });

  // `event.respondWith` wasn't called
  if (!deferred.done) {
    deferred.resolve(null);
  }

  return deferred.promise;
}
//# sourceMappingURL=dispatch-fetch-event.js.map