'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

/**
 * Parse response with the specified `responseType`,
 * return `null` if any error occurs, see:
 * https://xhr.spec.whatwg.org/#the-response-attribute
 *
 * @async
 * @param  {Response}     response     Response object
 * @param  {string}       responseType XHR responseType
 * @return {Promise<any>}
 */
var parseResponse = function () {
  var _ref2 = _asyncToGenerator(_regenerator2.default.mark(function _callee2(response, responseType) {
    var res, text, parser;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.prev = 0;
            res = response.clone();
            _context2.t0 = responseType;
            _context2.next = _context2.t0 === '' ? 5 : _context2.t0 === 'text' ? 5 : _context2.t0 === 'json' ? 8 : _context2.t0 === 'blob' ? 11 : _context2.t0 === 'arraybuffer' ? 14 : _context2.t0 === 'document' ? 17 : 22;
            break;

          case 5:
            _context2.next = 7;
            return res.text();

          case 7:
            return _context2.abrupt('return', _context2.sent);

          case 8:
            _context2.next = 10;
            return res.json();

          case 10:
            return _context2.abrupt('return', _context2.sent);

          case 11:
            _context2.next = 13;
            return res.blob();

          case 13:
            return _context2.abrupt('return', _context2.sent);

          case 14:
            _context2.next = 16;
            return res.arrayBuffer();

          case 16:
            return _context2.abrupt('return', _context2.sent);

          case 17:
            _context2.next = 19;
            return res.text();

          case 19:
            text = _context2.sent;
            parser = new DOMParser();
            return _context2.abrupt('return', parser.parseFromString(text, getDocumentMIME(res)));

          case 22:
            _context2.next = 26;
            break;

          case 24:
            _context2.prev = 24;
            _context2.t1 = _context2['catch'](0);

          case 26:
            return _context2.abrupt('return', null);

          case 27:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this, [[0, 24]]);
  }));

  return function parseResponse(_x2, _x3) {
    return _ref2.apply(this, arguments);
  };
}();

/**
 * Get MIME from 'content-type' header
 * @param  {Response} res Response object
 * @return {string}
 */


exports.patchXHR = patchXHR;

var _createEvent = require('./create-event');

var _dispatchFetchEvent = require('./dispatch-fetch-event');

var _utils = require('../../utils/');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Patch native `XMLHttpRequest`
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Notes:
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * - When and how to dispatch fetch event:
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   1. A XMLHttpRequest won't be sent until we call `xhr.send()`, so we should dispatch a
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   fetch event when `xhr.send()` is invoked,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   2. By overriding `xhr.open()` method, we can get `request_method` and `request_url`,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   3. By overriding `xhr.setRequestHeader()`, we can get `request_headers`,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   4. Create a request instance with `request_method`, `request_url`, `request_headers`,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *      and other options got from `this[xhrProp]` like `this.withCredentials` (will be
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *      delegated to `this.nativeXHR[xhrProp]`).
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   5. Dispatch fetch event with the request.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   6. If fetch event responds with a `Response`, parse it and dispatch `readystatechange`
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *      event and progress events via `this.dispatchEvent()` (will be delegated to
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *      `this.nativeXHR.dispatchEvent()`).
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   7. Or, re-send native requeust via `this.nativeXHR.send()`.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * - About events handling:
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   1. Since we delegate all `XMLHttpRequest.prototype` methods to `this.nativeXHR`, there's
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   no need to create an event emitter, simply calling `this.dispatchEvent()` and all
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   registered listeners will be invoked, including those are set by `instance#on[event]`.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   2. Meanwhile, all the event handlers you registered on `ExtandableXHR` instance will actually be
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   registered on `this.nativeXHR`. So even if we re-send the native request, all the handlers will
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   be called properly.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */

function patchXHR() {
  if (XMLHttpRequest.mockerPatched) {
    return;
  }

  self.XMLHttpRequest = MockerXHR;
}

// only `readystatechange` event and progress events are need to be dispatched
var EVENTS_LIST = ['readystatechange', 'loadstart', 'progress', 'load', 'loadend'];

var ExtandableXHR = (0, _utils.extensify)(XMLHttpRequest);

var MockerXHR = function (_ExtandableXHR) {
  _inherits(MockerXHR, _ExtandableXHR);

  function MockerXHR() {
    var _temp, _this, _ret;

    _classCallCheck(this, MockerXHR);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, _ExtandableXHR.call.apply(_ExtandableXHR, [this].concat(args))), _this), _this._requestHeaders = new Headers(), _this._responseHeaders = null, _this._responseMIME = undefined, _this._method = undefined, _this._url = undefined, _temp), _possibleConstructorReturn(_this, _ret);
  }
  // marked with `mockerPatched` symbol


  // keep a native reference


  // record request headers via `setRequestHeader` method


  // save response headers for `getResponseHeader(s)` methods


  // override response 'Content-Type' header via `overrideMimeType` method


  // save request method and url via `open` method


  MockerXHR.prototype.setRequestHeader = function setRequestHeader(header, value) {
    _ExtandableXHR.prototype.setRequestHeader.call(this, header, value);
    this._requestHeaders.append(header, value);
  };

  MockerXHR.prototype.getResponseHeader = function getResponseHeader(header) {
    if (!this._responseHeaders) {
      return _ExtandableXHR.prototype.getResponseHeader.call(this, header);
    }

    return this._responseHeaders.get(header);
  };

  MockerXHR.prototype.getAllResponseHeaders = function getAllResponseHeaders() {
    if (!this._responseHeaders) {
      return _ExtandableXHR.prototype.getAllResponseHeaders.call(this);
    }

    var results = [];

    // https://xhr.spec.whatwg.org/#dom-xmlhttprequest-getallresponseheaders
    var seperator = String.fromCharCode(0x3A) + String.fromCharCode(0x20);
    var linebreaker = String.fromCharCode(0x0D) + String.fromCharCode(0x0A);

    this._responseHeaders.forEach(function (value, name) {
      results.push([name, value].join(seperator));
    });

    return results.join(linebreaker);
  };

  MockerXHR.prototype.overrideMimeType = function overrideMimeType(mime) {
    /* istanbul ignore if */
    if (!_ExtandableXHR.prototype.overrideMimeType) {
      return;
    }

    _ExtandableXHR.prototype.overrideMimeType.call(this, mime);
    this._responseMIME = mime;
  };

  MockerXHR.prototype.open = function open(method, url) {
    var _ExtandableXHR$protot;

    for (var _len2 = arguments.length, rest = Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
      rest[_key2 - 2] = arguments[_key2];
    }

    (_ExtandableXHR$protot = _ExtandableXHR.prototype.open).call.apply(_ExtandableXHR$protot, [this, method, url].concat(rest));
    this._method = method;
    this._url = url;
  };

  MockerXHR.prototype.send = function send(data) {
    var _this2 = this;

    if (this.readyState !== this.OPENED) {
      throw new Error('Failed to execute \'send\' on \'XMLHttpRequest\': The object\'s state must be OPENED.');
    }

    this._mockFetch(data).then(function (result) {
      if (result) {
        // `event.respondWith` called
        // resolve with mock response
        _this2._processResponse(result);
      } else {
        // send real XMLHttpRequest
        _ExtandableXHR.prototype.send.call(_this2, data);
      }
    });
  };

  /**
   * Mock fetch
   * @private
   * @param  {any}               data request body
   * @return {Promise<Response>}
   */


  MockerXHR.prototype._mockFetch = function _mockFetch(data) {
    // GET|HEAD requests cannot include body
    // set body to `null` will raise a TypeMismatchError in IE Edge
    var body = this._method === 'GET' || this._method === 'HEAD' ? undefined : data;

    var credentials = this.withCredentials ? 'include' : 'omit';

    var request = new Request(this._url, {
      body: body,
      credentials: credentials,
      method: this._method,
      headers: this._requestHeaders
    });

    return (0, _dispatchFetchEvent.dispatchFetchEvent)(request);
  };

  /**
   * Process response object
   *
   * @private
   * @param  {Response}      response Response object
   */


  MockerXHR.prototype._processResponse = function () {
    var _ref = _asyncToGenerator(_regenerator2.default.mark(function _callee(response) {
      var result;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (this._responseMIME) {
                // apply `overrideMimeType`
                response.headers.set('content-type', this._responseMIME);
              }

              _context.next = 3;
              return parseResponse(response, this.responseType);

            case 3:
              result = _context.sent;


              this._responseHeaders = response.headers;

              // pretend this request is DONE
              this._setProperty('readyState', this.DONE);
              this._setProperty('responseURL', response.url);
              this._setProperty('status', response.status);
              this._setProperty('statusText', response.statusText);
              this._setProperty('response', result);

              if (!this.responseType || this.responseType === 'text') {
                this._setProperty('responseText', result);
              }

              if (this.responseType === 'document') {
                this._setProperty('responseXML', result);
              }

              this._dispatchEvents();

            case 13:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function _processResponse(_x) {
      return _ref.apply(this, arguments);
    }

    return _processResponse;
  }();

  /**
   * Assign property to self
   * @private
   * @param {string} name  property name
   * @param {any}    value property value
   */


  MockerXHR.prototype._setProperty = function _setProperty(name, value) {
    // in IE & Safari, these property are internally read-only on native XHR instance
    // assign to patched XHR, as a trade-off
    Object.defineProperty(this, name, {
      value: value,
      writable: false,
      enumerable: true,
      configurable: true
    });
  };

  /**
   * Dispatch XHR events
   * @private
   */


  MockerXHR.prototype._dispatchEvents = function _dispatchEvents() {
    var _this3 = this;

    EVENTS_LIST.forEach(function (type) {
      var event = (0, _createEvent.createEvent)(type);

      if (type !== 'readystatechange') {
        // progress event
        event.total = event.loaded = 1;
      }

      // Caveat: `this` & `event.target` are still the native one
      _this3.dispatchEvent(event);
    });
  };

  return MockerXHR;
}(ExtandableXHR);

MockerXHR.mockerPatched = true;
MockerXHR.native = XMLHttpRequest;
function getDocumentMIME(res) {
  var contentType = res.headers.get('content-type');

  /* istanbul ignore if */
  if (!contentType) {
    return 'text/html';
  }

  // strip charset
  return contentType.replace(/;.*/, '');
}
//# sourceMappingURL=patch-xhr.js.map