'use strict';

exports.__esModule = true;
exports.ModernClient = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _utils = require('../../utils/');

var _register = require('./register');

var _connect = require('./connect');

var _getNewestReg = require('./get-newest-reg');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ModernClient = exports.ModernClient = function () {

  /**
   * Defines whether a client has connected to mocker server.
   *
   * @readonly
   * @type {Promise<ServiceWorkerRegistration>}
   */
  function ModernClient(scriptURL) {
    var _this = this;

    _classCallCheck(this, ModernClient);

    this.isLegacy = false;
    this.ready = null;
    this.controller = null;

    /* istanbul ignore next */
    this.ready = this._init(scriptURL).then(function (registration) {
      _this.controller = registration.active;
      return registration;
    }).catch(function (error) {
      _utils.debug.error('mocker initialization failed: ', error);
    });
  }

  /**
   * Update the service worker registration immediately
   *
   * @return {Promise<ServiceWorkerRegistration>}
   */


  /**
   * Points to currently activated ServiceWorker object.
   *
   * @readonly
   * @type {ServiceWorker}
   */

  /**
   * Indicates which mode current client is running on.
   *
   * @readonly
   * @type {boolean}
   */


  ModernClient.prototype.update = function () {
    var _ref = _asyncToGenerator(_regenerator2.default.mark(function _callee() {
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              return _context.abrupt('return', (0, _getNewestReg.getNewestReg)());

            case 1:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function update() {
      return _ref.apply(this, arguments);
    }

    return update;
  }();

  /**
   * Get current service worker registration.
   *
   * @return {Promise<ServiceWorkerRegistration>}
   */


  ModernClient.prototype.getRegistration = function () {
    var _ref2 = _asyncToGenerator(_regenerator2.default.mark(function _callee2() {
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              return _context2.abrupt('return', this.ready);

            case 1:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    function getRegistration() {
      return _ref2.apply(this, arguments);
    }

    return getRegistration;
  }();

  /* istanbul ignore next: don't unregister sw in tests */
  /**
   * Unregister current service worker registration,
   * call this method will invoke `ServiceWorkerRegistration.unregister()` method when possible.
   *
   * @return {Promise<boolean>}
   */


  ModernClient.prototype.unregister = function () {
    var _ref3 = _asyncToGenerator(_regenerator2.default.mark(function _callee3() {
      var registration, result;
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return this.getRegistration();

            case 2:
              registration = _context3.sent;
              _context3.next = 5;
              return registration.unregister();

            case 5:
              result = _context3.sent;


              if (!result) {
                // tslint:disable-next-line max-line-length
                _utils.debug.warn('this service worker has already been unregistered, you may need to close all relative tabs to remove it');
              }

              return _context3.abrupt('return', result);

            case 8:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, this);
    }));

    function unregister() {
      return _ref3.apply(this, arguments);
    }

    return unregister;
  }();

  /**
   * Init mocker
   *
   * @private
   * @param  {string} scriptURL
   * @return {Promise<ServiceWorkerRegistration>}
   */


  ModernClient.prototype._init = function () {
    var _ref4 = _asyncToGenerator(_regenerator2.default.mark(function _callee4(scriptURL) {
      var registration;
      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              _context4.next = 2;
              return (0, _register.register)(scriptURL, {
                scope: location.pathname
              });

            case 2:
              registration = _context4.sent;


              this._autoSyncClient();

              return _context4.abrupt('return', registration);

            case 5:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, this);
    }));

    function _init(_x) {
      return _ref4.apply(this, arguments);
    }

    return _init;
  }();

  /**
   * Synchronize mocker controller when sw controller changed
   *
   * @private
   */


  ModernClient.prototype._autoSyncClient = function _autoSyncClient() {
    var _this2 = this;

    var _navigator = navigator,
        serviceWorker = _navigator.serviceWorker;


    var updateLog = _utils.debug.scope('update');

    /* istanbul ignore next: won't occur in tests */
    serviceWorker.addEventListener('controllerchange', _asyncToGenerator(_regenerator2.default.mark(function _callee5() {
      var registration;
      return _regenerator2.default.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              _context5.prev = 0;
              _context5.next = 3;
              return (0, _connect.connect)(true);

            case 3:
              registration = _context5.sent;

              _this2.controller = registration.active;

              updateLog.color('crimson').warn('mocker updated, reload your requests to take effect');
              _context5.next = 11;
              break;

            case 8:
              _context5.prev = 8;
              _context5.t0 = _context5['catch'](0);

              updateLog.error('connecting to new service worker failed', _context5.t0);

            case 11:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, _this2, [[0, 8]]);
    })));
  };

  return ModernClient;
}();
//# sourceMappingURL=client.js.map