'use strict';

exports.__esModule = true;
exports.connect = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

/**
 * Connect to service worker
 *
 * @param  {boolean} [skipUpdate=false] Set to `true` to bypass auto update
 * @return {Promise<ServiceWorkerRegistration>}
 */
var connect = exports.connect = function () {
  var _ref = _asyncToGenerator(_regenerator2.default.mark(function _callee() {
    var skipUpdate = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

    var _navigator, serviceWorker, hasController, reg;

    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _navigator = navigator, serviceWorker = _navigator.serviceWorker;

            // controller may be set when sw is ready

            hasController = !!serviceWorker.controller;

            // chrome will sometimes be hanging after reloading page
            // delay all actions until sw is ready

            _context.next = 4;
            return serviceWorker.ready;

          case 4:
            if (!(!skipUpdate && hasController)) {
              _context.next = 10;
              break;
            }

            _context.next = 7;
            return (0, _getNewestReg.getNewestReg)();

          case 7:
            _context.t0 = _context.sent;
            _context.next = 13;
            break;

          case 10:
            _context.next = 12;
            return serviceWorker.getRegistration();

          case 12:
            _context.t0 = _context.sent;

          case 13:
            reg = _context.t0;
            return _context.abrupt('return', handshake(reg));

          case 15:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function connect() {
    return _ref.apply(this, arguments);
  };
}();

/**
 * Establish service worker connection
 *
 * @param  {ServiceWorkerRegistration} registration
 * @return {Promise<ServiceWorkerRegistration>}
 */


var handshake = function () {
  var _ref2 = _asyncToGenerator(_regenerator2.default.mark(function _callee2(registration) {
    var controller;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            controller = registration.active;

            /* istanbul ignore if */

            if (controller) {
              _context2.next = 3;
              break;
            }

            throw new Error('no active service worker registration is found');

          case 3:
            if (navigator.serviceWorker.controller) {
              _context2.next = 6;
              break;
            }

            _context2.next = 6;
            return (0, _utils.sendMessageRequest)(controller, {
              action: _constants.ACTION.REQUEST_CLAIM
            });

          case 6:
            _context2.next = 8;
            return (0, _utils.sendMessageRequest)(controller, {
              action: _constants.ACTION.PING
            });

          case 8:

            _utils.debug.scope('modern').info('connection established');

            return _context2.abrupt('return', registration);

          case 10:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function handshake(_x2) {
    return _ref2.apply(this, arguments);
  };
}();

var _utils = require('../../utils/');

var _constants = require('../../constants/');

var _getNewestReg = require('./get-newest-reg');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }
//# sourceMappingURL=connect.js.map