'use strict';

exports.__esModule = true;
exports.MockerRequest = exports.ExtandableRequest = undefined;

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

var _utils = require('../utils/');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var requestLog = _utils.debug.scope('request');

// convert native `Request` to extendable
// export or you'll get an error of 'using private'
/* istanbul ignore next: polyfill is legacy browsers only */
var ExtandableRequest = exports.ExtandableRequest = fetch.polyfill ? Request : (0, _utils.extensify)(Request);

var MockerRequest = exports.MockerRequest = function (_ExtandableRequest) {
  _inherits(MockerRequest, _ExtandableRequest);

  /**
   * Constructs a mocker request object
   *
   * @param {FetchEvent} event Fetch event
   * @param {RouteRule}  route Router rule object
   */


  /**
   * Fetch event
   *
   * @private
   * @readonly
   * @type {FetchEvent}
   */


  /**
   * Contains properties mapped the route parameters.
   *
   * @readonly
   * @type {object}
   */

  /**
   * Literally equivalent to `router.baseURL` property of current router.
   *
   * @readonly
   * @type {string}
   */
  function MockerRequest(event, route) {
    _classCallCheck(this, MockerRequest);

    var request = event.request;
    var regex = route.regex,
        keys = route.keys,
        baseURL = route.baseURL;

    // avoid polluting original request

    var _this = _possibleConstructorReturn(this, _ExtandableRequest.call(this, request.clone()));

    _this.baseURL = undefined;
    _this.path = undefined;
    _this.params = null;
    _this.query = null;
    _this._event = null;
    _this._route = null;


    var base = new URL(baseURL);
    var requestURL = new URL(request.url, location.href);
    var path = base.pathname === '/' ? requestURL.pathname : requestURL.pathname.replace(base.pathname, '');

    var matches = regex.exec(path);
    var params = {};

    // skip full matched string at [0]
    var max = matches.length;
    for (var i = 1; i < max; i++) {
      var name = keys[i - 1].name;

      params[name] = decodeParam(matches[i]);
    }

    _this._event = event;
    _this._route = route;

    _this.path = path;
    _this.params = params;
    _this.baseURL = baseURL;
    _this.query = _qs2.default.parse(requestURL.search.slice(1)); // remove leading '?'

    // overwrite relative URL from fetch polyfill
    /* istanbul ignore if: legacy browsers only */
    if (_this.url !== requestURL.href) {
      Object.defineProperty(_this, 'url', {
        value: requestURL.href,
        writable: false,
        enumerable: true,
        configurable: true
      });
    }
    return _this;
  }

  /**
   * Route rule object
   *
   * @private
   * @readonly
   * @type {RouteRule}
   */


  /**
   * Contains a property for each query string parameter from the request.
   *
   * @readonly
   * @type {object}
   */


  /**
   * Contains the path part of the current request.
   *
   * @readonly
   * @type {string}
   */


  MockerRequest.prototype.clone = function clone() {
    return new MockerRequest(this._event, this._route);
  };

  return MockerRequest;
}(ExtandableRequest);

/**
 * Decode request parameter
 *
 * @param  {string} value Parameter value
 * @return {string}
 */


function decodeParam(value) {
  if (typeof value !== 'string' || value.length === 0) {
    return value;
  }

  try {
    return decodeURIComponent(value);
  } catch (err) {
    requestLog.error('decode param: ' + value + ' failed', err);

    return null;
  }
}
//# sourceMappingURL=request.js.map