'use strict';

exports.__esModule = true;
exports.MockerResponse = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Concat new request info with given old request
 */
/**
 * Concat new request info with given old request
 *
 * @param  {Request}          oldRequest Old Request object
 * @param  {RequestInfo}      input      New Request input
 * @param  {RequestInit}      [init={}]  New Request init
 * @return {Promise<Request>}
 */
var concatRequest = function () {
  var _ref2 = _asyncToGenerator(_regenerator2.default.mark(function _callee2(oldRequest, input) {
    var init = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    var tempRequest;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            tempRequest = new Request(input, oldRequest);

            if (!(!init.body && oldRequest.method !== 'GET' && oldRequest.method !== 'HEAD')) {
              _context2.next = 5;
              break;
            }

            _context2.next = 4;
            return bodyParser(oldRequest);

          case 4:
            init.body = _context2.sent;

          case 5:
            return _context2.abrupt('return', new Request(tempRequest, init));

          case 6:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function concatRequest(_x4, _x5) {
    return _ref2.apply(this, arguments);
  };
}();

/**
 * Parse request body
 * 1. If you're using github fetch polyfill, return the private member `_bodyInit`,
 * 2. Else parsing request body as blob.
 *
 * @param  {Request} request Request object to be parsed
 * @return {any}
 */


var _mimeComponent = require('mime-component');

var _mimeComponent2 = _interopRequireDefault(_mimeComponent);

var _statuses = require('statuses');

var _statuses2 = _interopRequireDefault(_statuses);

var _utils = require('../utils/');

var _request = require('./request');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var responseLog = _utils.debug.scope('response');
var IS_IE_EDGE = /Edge/.test(navigator.userAgent);

var MockerResponse = function () {

  /**
   * Constructs a mocker response object
   *
   * @param {FetchEvent} event Fetch event
   */


  /**
   * Response status code
   *
   * @private
   * @type {number}
   */


  /**
   * Fetch event
   *
   * @private
   * @readonly
   * @type {FetchEvent}
   */
  function MockerResponse(event) {
    _classCallCheck(this, MockerResponse);

    this.headers = new Headers({
      'X-Powered-By': 'ServiceMocker'
    });
    this._event = null;
    this._body = undefined;
    this._statusCode = 200;
    this._deferred = new _utils.Defer();
    var _deferred = this._deferred;


    this._event = event;

    // everything within service worker should be asynchronous
    event.respondWith(_deferred.promise);
  }

  /**
   * Sets the HTTP status for the response.
   *
   * @chainable
   * @param  {number} code Status code
   * @return {this}
   */


  /**
   * Internal defer object to resolve response
   *
   * @private
   * @readonly
   * @type {Defer}
   */


  /**
   * Response body to be sent
   *
   * @private
   * @type {any}
   */

  /**
   * Response headers
   *
   * @readonly
   * @type {Headers}
   */


  MockerResponse.prototype.status = function status(code) {
    this._statusCode = code;

    return this;
  };

  /**
   * Sets the Content-Type HTTP header to the MIME type.
   * If the given MIME doesn't contain '/' character,
   * use `mime.lookup(type)` to obtain MIME type.
   *
   * @chainable
   * @param  {string} type MIME type
   * @return {this}
   */


  MockerResponse.prototype.type = function type(_type) {
    var contentType = _type.indexOf('/') === -1 ? _mimeComponent2.default.lookup(_type) : _type;

    this.headers.set('content-type', contentType);

    return this;
  };

  /**
   * Send a JSON response.
   *
   * @param {any} [body] Any JSON compatible type, including object, array, string, Boolean, or number.
   */


  MockerResponse.prototype.json = function json(body) {
    this._body = JSON.stringify(body);

    if (!this.headers.has('content-type')) {
      this.type('json');
    }

    this.end();
  };

  /**
   * Sends the HTTP response.
   *
   * @param {any} [body] Response body, one of Blob, ArrayBuffer, Object, or any primitive types
   */


  MockerResponse.prototype.send = function send(body) {
    // don't parse native Response objects
    if (body instanceof Response) {
      this._body = body;
      this.end();
      return;
    }

    var contentType = 'text';

    switch (typeof body === 'undefined' ? 'undefined' : _typeof(body)) {
      case 'string':
        // follow the express style
        this._body = body;
        contentType = 'html';
        break;

      case 'boolean':
      case 'number':
      case 'object':
        if (body instanceof Blob) {
          this._body = body;
          contentType = body.type || 'bin';
        } else if (body instanceof ArrayBuffer) {
          this._body = body;
          contentType = 'bin';
        } else {
          this.json(body);
          return;
        }
        break;
    }

    if (!this.headers.has('content-type')) {
      this.type(contentType);
    }

    this.end();
  };

  /**
   * Set the response HTTP status code to statusCode and
   * send its status text representation as the response body.
   *
   * Equivalent to `res.status(code).send(statusText)`
   *
   * @param {number} code Status code
   */


  MockerResponse.prototype.sendStatus = function sendStatus(code) {
    var body = _statuses2.default[code] || JSON.stringify(code);

    this.type('text').status(code).send(body);
  };

  /**
   * End the response processing and pass the response to `fetchEvent.respondWith()`.
   * Simply call this method will end the response WITHOUT any data,
   * if you want to respond with data, use `res.send()` and `res.json()`.
   */


  MockerResponse.prototype.end = function end() {
    // respond with body if it's a native Response object
    if (this._body instanceof Response) {
      this._deferred.resolve(this._body);
      return;
    }

    var request = this._event.request;


    var responseBody = this._body;

    // leave body empty for null body status
    if (_statuses2.default.empty[this._statusCode]) {
      /* istanbul ignore if */
      if (IS_IE_EDGE) {
        responseLog.warn('using null body status in IE Edge may raise a `TypeMismatchError` Error');
      }

      responseBody = undefined;
    }

    // skip body for HEAD requests
    if (request.method === 'HEAD') {
      responseBody = undefined;
    }

    // set default contentType to 'text/plain', see
    // https://tools.ietf.org/html/rfc2045#section-5.2
    if (!this.headers.has('content-type')) {
      this.type('text');
    }

    var responseInit = {
      headers: this.headers,
      status: this._statusCode,
      statusText: _statuses2.default[this._statusCode] || JSON.stringify(this._statusCode)
    };

    var response = new Response(responseBody, responseInit);

    this._deferred.resolve(response);
  };

  /**
   * Forward the request to another destination.
   * The forwarded request will NOT be captured by service worker.
   *
   * @param {RequestInfo|MockerRequest} input Destination URL or a Request object or MockerRequest
   * @param {RequestInit=}              init  Fetch request init
   */


  MockerResponse.prototype.forward = function () {
    var _ref = _asyncToGenerator(_regenerator2.default.mark(function _callee(input, init) {
      var request;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              request = void 0;

              if (!(input instanceof Request)) {
                _context.next = 5;
                break;
              }

              // forward native Request
              request = new Request(input, init);
              _context.next = 12;
              break;

            case 5:
              if (!(input instanceof _request.MockerRequest)) {
                _context.next = 9;
                break;
              }

              // forward MockerRequest
              request = new Request(input._native, init);
              _context.next = 12;
              break;

            case 9:
              _context.next = 11;
              return concatRequest(this._event.request, input, init);

            case 11:
              request = _context.sent;

            case 12:

              // fetch will somehow consume the body
              this._deferred.resolve(nativeFetch(request));

            case 13:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function forward(_x, _x2) {
      return _ref.apply(this, arguments);
    }

    return forward;
  }();

  return MockerResponse;
}();

exports.MockerResponse = MockerResponse;
function bodyParser(request) {
  // handle github fetch polyfill
  /* istanbul ignore if */
  if (fetch.polyfill) {
    return request._bodyInit;
  }

  try {
    // always parse as blob
    return request.clone().blob();
  } catch (e) {
    /* istanbul ignore next */
    responseLog.warn('parsing request body failed, you may need to parse it manually', e);
  }
}

/* istanbul ignore next */
/**
 * Fetch with native `fetch`
 * 1. If `fetch.mockerPatched` is found, it means you're running on
 *    legacy mode with fetch support, return with `fetch.native`.
 *
 * 2. Else if `XMLHttpRequest.mockerPatched` is found, you're possibly
 *    using a fetch polyfill, processing as the following:
 *    2.1. Reset `XMLHttpRequest` to native one `(patched)XMLHttpRequest.native`,
 *    2.2. Run fetch polyfill (with native XHR),
 *    2.3. Restore `XMLHttpRequest` to patched one,
 *    2.4. Return the fetch promise.
 *
 * 3. Or, you may be running in service worker context, return `fetch`.
 *
 * @param  {RequestInfo}       input
 * @param  {RequestInit}       [init]
 * @return {Promise<Response>}
 */
function nativeFetch(input, init) {
  var _self = self,
      fetch = _self.fetch,
      XHR = _self.XMLHttpRequest;

  // native fetch

  if (fetch.mockerPatched) {
    return fetch.native(input, init);
  }

  // fetch polyfills
  if (XHR && XHR.mockerPatched) {
    // restore native...
    self.XMLHttpRequest = XHR.native;
    // do a native fetch
    var promise = fetch(input, init);
    // replace with our fetch
    self.XMLHttpRequest = XHR;

    return promise;
  }

  return fetch(input, init);
}
//# sourceMappingURL=response.js.map