'use strict';

exports.__esModule = true;
exports.MockerServer = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _router = require('./router');

var _constants = require('../constants/');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var MockerServer = exports.MockerServer = function () {

  /**
   * Constructs a server object
   *
   * @param {string} [baseURL='/'] The base URL of all routes, default is '/'
   */

  /**
   * Indicates which mode current server is running on
   *
   * @readonly
   * @type {boolean}
   */
  function MockerServer() {
    var baseURL = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '/';

    _classCallCheck(this, MockerServer);

    this.isLegacy = self === self.window;
    this.router = null;

    this.router = new _router.MockerRouter(baseURL);
  }

  /**
   * Attach middleware to current server
   *
   * @param  {MiddlewareFn} fn Middleware function
   * @return {this}
   */


  /**
   * Returns the Router instance of current server
   *
   * @readonly
   * @type {MockerRouter}
   */


  MockerServer.prototype.use = function use(fn) {
    this.router.use(fn);

    return this;
  };

  return MockerServer;
}();

// Event listeners MUST be added on the initial evaluation of worker scripts.
/* istanbul ignore next: unable to report coverage from sw context */


self.addEventListener('message', function () {
  var _ref = _asyncToGenerator(_regenerator2.default.mark(function _callee(event) {
    var data, ports, port;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            data = event.data, ports = event.ports;

            if (!(!data || !ports.length)) {
              _context.next = 3;
              break;
            }

            return _context.abrupt('return');

          case 3:
            port = ports[0];

            // handle connections

            _context.t0 = data.action;
            _context.next = _context.t0 === _constants.ACTION.PING ? 7 : _context.t0 === _constants.ACTION.REQUEST_CLAIM ? 8 : 11;
            break;

          case 7:
            return _context.abrupt('return', port.postMessage({
              action: _constants.ACTION.PONG
            }));

          case 8:
            _context.next = 10;
            return self.clients.claim();

          case 10:
            return _context.abrupt('return', port.postMessage({
              action: _constants.ACTION.ESTABLISHED
            }));

          case 11:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function (_x2) {
    return _ref.apply(this, arguments);
  };
}());

self.addEventListener('fetch', function (event) {
  _router.MockerRouter.routers.some(function (router) {
    return router._match(event);
  });
});

// IE will somehow fires `activate` event on form elements
/* istanbul ignore if: unable to report coverage from sw context */
if (self !== self.window) {
  self.addEventListener('install', function (event) {
    event.waitUntil(self.skipWaiting());
  });

  self.addEventListener('activate', function (event) {
    event.waitUntil(self.clients.claim());
  });
}
//# sourceMappingURL=server.js.map