'use strict';

exports.__esModule = true;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var isIE = /Trident|Edge/.test(navigator.userAgent);

var defaultColor = 'dodgerblue';

var colorPresets = {
  log: defaultColor,
  info: defaultColor,
  warn: 'goldenrod',
  error: 'crimson'
};

var PrefixedConsole = exports.PrefixedConsole = function () {
  /**
   * Logger namespace
   *
   * @private
   * @type {string}
   */
  function PrefixedConsole() {
    var namespace = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'mocker';
    var color = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : colorPresets.log;

    _classCallCheck(this, PrefixedConsole);

    this._namespace = 'mocker';
    this._color = colorPresets.log;

    this._namespace = namespace;
    this._color = color;
  }

  /**
   * Set logger color, returns new PrefixedConsole
   *
   * @param  {string} c Color string
   * @return {PrefixedConsole}
   */


  /**
   * Logger color
   *
   * @private
   * @type {string}
   */


  PrefixedConsole.prototype.color = function color(c) {
    return new PrefixedConsole(this._namespace, c);
  };

  /**
   * Set logger namespace, returns new PrefixedConsole
   *
   * @param  {string} ns Namespace
   * @return {PrefixedConsole}
   */


  PrefixedConsole.prototype.scope = function scope(ns) {
    return new PrefixedConsole(this._namespace + ':' + ns, this._color);
  };

  return PrefixedConsole;
}();

var debug = exports.debug = new PrefixedConsole();

/* istanbul ignore else */
// inherit console methods
if (typeof Object.setPrototypeOf === 'function') {
  Object.setPrototypeOf(PrefixedConsole.prototype, console);
} else {
  var desc = {};

  Object.getOwnPropertyNames(PrefixedConsole.prototype).forEach(function (prop) {
    desc[prop] = Object.getOwnPropertyDescriptor(PrefixedConsole.prototype, prop);
  });

  PrefixedConsole.prototype = Object.create(console, desc);
}

['log', 'info', 'warn', 'error'].forEach(function (method) {
  PrefixedConsole.prototype[method] = function logger() {
    var _console2;

    var _namespace = this._namespace,
        _color = this._color;

    /* istanbul ignore if */

    for (var _len = arguments.length, messages = Array(_len), _key = 0; _key < _len; _key++) {
      messages[_key] = arguments[_key];
    }

    if (isIE) {
      var _console;

      return (_console = console)[method].apply(_console, ['[' + _namespace + ']'].concat(messages));
    }

    var head = '%c[' + _namespace + ']%c';

    if (/%c/.test(messages[0])) {
      head = head + ' ' + messages.shift();
    }

    var color = _color === defaultColor ? colorPresets[method] : _color;

    (_console2 = console)[method].apply(_console2, [head, 'color: ' + color, 'color: #000'].concat(messages));
  };
});
//# sourceMappingURL=debug.js.map