'use strict';

exports.__esModule = true;
exports.sendMessageRequest = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Send message and get the response
 *
 * @param  {any}    target  Message target
 * @param  {any}    message Message body
 * @param  {number} timeout Messaging timeout
 * @return {Promise<any>} Resolves with response message
 */
var sendMessageRequest = exports.sendMessageRequest = function () {
  var _ref = _asyncToGenerator(_regenerator2.default.mark(function _callee(target, message) {
    var timeout = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 3 * 1e3;

    var _ref2, port1, port2;

    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _ref2 = new MessageChannel(), port1 = _ref2.port1, port2 = _ref2.port2;
            return _context.abrupt('return', new Promise(function (resolve, reject) {
              var timer = isFinite(timeout) && setTimeout(function () {
                reject(new Error('messaging timeout: ' + JSON.stringify(message)));
              }, timeout);

              port1.onmessage = function (_ref3) {
                var data = _ref3.data;

                if (timer) {
                  clearTimeout(timer);
                }

                // avoid high transient memory usage, see
                // https://html.spec.whatwg.org/multipage/comms.html#ports-and-garbage-collection
                port1.close();
                port2.close();

                /* istanbul ignore else */
                if ((typeof data === 'undefined' ? 'undefined' : _typeof(data)) === 'object') {
                  data.request = message;
                }

                if (data && data.error) {
                  reject(data);
                } else {
                  resolve(data);
                }
              };

              if (target === self.window) {
                // posting message to self => legacy mode
                // add `origin` param to `window.postMessage(message, targetOrigin, [transfer])`
                target.postMessage(message, '*', [port2]);
              } else {
                target.postMessage(message, [port2]);
              }
            }));

          case 2:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function sendMessageRequest(_x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }
//# sourceMappingURL=send-message-request.js.map